package com.digiwin.athena.app.service.staff;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.common.enums.OptionEnums;
import com.digiwin.athena.app.infra.common.enums.StatusEnums;
import com.digiwin.athena.app.infra.common.utils.BeanCopyUtil;
import com.digiwin.athena.app.infra.dto.staff.StaffDTO;
import com.digiwin.athena.app.infra.entity.StaffEntity;
import com.digiwin.athena.app.infra.repository.StaffRepository;
import com.digiwin.athena.app.infra.service.StaffService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * @Auther: zhenggl
 * @Date: 2023/4/28
 */
@Service
@Log4j2
public class StaffUpdateEAIService extends AbsEAIService {

    @Resource
    private StaffRepository staffRepository;

    @Resource
    private StaffService staffService;

    @Override
    public String getServiceName() {
        return StaffUtil.DEMO_WAIT_REGISTER_PERSONNEL_INFO_UPDATE;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {
        //反序列化字段
        EAIRequest request = new EAIRequest(messageBody);
        String option = request.getObject("option", String.class);
        List<StaffDTO> staffList = request.getObject("wait_register_personnel", StaffUtil.LIST_DTO_STAFF);
        //界面
        if (OptionEnums.BASIC.getValue().equals(option)){
            if (!CollectionUtils.isEmpty(staffList)){
                List<StaffEntity> staffEntities = BeanCopyUtil.copyList(staffList, StaffEntity.class);
                //区分是否有id没有则新增
                Map<Boolean, List<StaffEntity>> collect = staffEntities.stream().collect(Collectors.groupingBy(item -> StringUtils.isEmpty(item.getId())));
                List<StaffEntity> insert = collect.get(true);
                staffService.saveBatch(insert);
                List<StaffEntity> update = collect.get(false);
                staffRepository.updateStaffList(update, SecurityUtil.getUserProfile().getTenantSid());
            }
        }else if (OptionEnums.TASK.getValue().equals(option)){
            if (!CollectionUtils.isEmpty(staffList)){
                List<StaffEntity> staffEntities = BeanCopyUtil.copyList(staffList, StaffEntity.class);
                staffRepository.updateStatusAndDate(staffEntities,SecurityUtil.getUserProfile().getTenantSid());
            }
        }else if (OptionEnums.FLOW.getValue().equals(option)){
            List<Long> idList = staffList.stream().map(StaffDTO::getId).collect(Collectors.toList());
            LambdaUpdateWrapper<StaffEntity> lmu = new LambdaUpdateWrapper<>();
            lmu.set(StaffEntity::getStatus, StatusEnums.REPORTED.getValue());
            lmu.in(StaffEntity::getId,idList);
            staffService.update(lmu);
        }

        return EAIUtil.buildEAIResult(new HashMap<>());
    }
}
